#!/usr/bin/perl

#
#    PerlSquish - A perl script that converts z80 object files to TI-83
#                 program files
#    Copyright (C) 1999  Ben Lukens <ben@jbic.com>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# Read in the object file
$prog=join("",<>);

# If it's a Mac, tell MacPerl to quit when done.
if ("$^O" eq "MacOS") {
 MacPerl::Quit(3);
}

# Verify its an object file, and strip off line feeds and
# first 9 and last 2 chars of every line
$prog=~s/:.{8}([0-9a-fA-F]*?).{2}[\r\n]+/$1/gm or &error("This is not an IntelHex format object file.");

# Get the program's name
$name=&ask("Enter the program's name:");
&error("You must enter a program name.") if "$name" eq "";
&error("Name is too long. It must be under 8 characters.") if length("$name")>8;
$name=~s/\.83p$//i; # Strip off .83p if it was added to name
$out="$name.83p"; # Add .83p to output file name

# Get the program's comment
$comment=&ask("Enter the program's comment:");
&error("Comment is too long. It must be under 42 characters.") if length("$comment")>42;

# Squish if necessary
if (&answer("Do you want to squish this program (for SOS, ASHELL, or ZASMLOAD)?")) {
 $prog=pack("H*","$prog");
} else {
 $prog.=pack("H*","3FD43F303030303FD4"); # Add End 0000 End to unsquished progs
}

# Protect if necessary
if (&answer("Do you want this program to be protected?")) {
 $hidden="06";
} else {
 $hidden="05";
}

$head=pack("H*","2A2A544938332A2A1A0A00"); # Set the program head to **TI83**

$comment=pack("a42","$comment"); # Pack the comment, null padded
$name=pack("a8","$name"); # Pack the name, null padded
$headerlength=pack("H4","0B00"); # Pack header length (always 0B00)
$thelength=length($prog); # Get the length of the prog
$filelength=&word($thelength+17); # File length is thelength + 17 as a word
$datalength=&word($thelength+2); # Data length is thelength + 2 as a word
$proglength=&word($thelength); # Program length is thelength as a word
$progtype=pack("H2",$hidden); # Pack the protected status (05 or 06)
# Generate the checksum
$checksum=&word(unpack("%32C*","$headerlength$datalength$progtype$name$datalength$proglength$prog"));

# Save the program to a file
open(OUT,">$out");
binmode(OUT); # Bah, DOS pisses me off...
select OUT;
print "$head";
print "$comment";
print "$filelength";
print "$headerlength";
print "$datalength";
print "$progtype";
print "$name";
print "$datalength";
print "$proglength";
print "$prog";
print "$checksum";
close(OUT);

# Set the file type and creator on Macs
if ("$^O" eq "MacOS") {
 MacPerl::SetFileInfo("TI83","83P ","$out");
}

# Make a word: take in a decimal number, swap bytes, and pack
sub word {
 $_=$_[0];
 my $a=$_>>8;
 my $b=$_ & 255;
 return pack("c2",$b,$a);
}

# Ask for a string
sub ask {
 $_="$_[0]";
 my $input;
 if ("$^O" eq "MacOS") {
  return MacPerl::Ask("$_");
 } else {
  print "$_ ";
  chop($input=<STDIN>);
  return "$input";
 }
}

# Print error and die
sub error {
 $_="$_[0]";
 my $input;
 if ("$^O" eq "MacOS") {
  MacPerl::Answer("$_","Quit");
  exit 1;
 } else {
  die "$_\n";
 }
}

# Ask yes or no
sub answer {
 $_="$_[0]";
 my $input;
 if ("$^O" eq "MacOS") {
  return MacPerl::Answer("$_","Yes","No");
 } else {
  print "$_ (y/n) ";
  $input=<STDIN>;
  if ($input=~/^y/i) {
   return 1;
  } else {
   return 0;
  }
 }
}
